//------------------------------------------------------------------------------
//	dictoPop
//------------------------------------------------------------------------------

// strExt is the prefix of each class in this 'Ext'ension
const defTemplatePath = "chrome://dictoPop/skin/";
const strExtPrefix = "dictoPop";
const strExtId = strExtPrefix + "Id";
const strExtAnchorId = strExtPrefix + "AnchorId";
const strExtMessageId = strExtPrefix + "MessageId";
const strExtAnchor = strExtPrefix + "Anchor";
const strExtMessage = strExtPrefix + "Message";
const strExtTitleBar = strExtPrefix + "TitleBar";
const strExtStyles = strExtPrefix + ".css";

const strLanguageMapPrefix = "dp.languages.";
const strDictionaryMapPrefix = "dp.label.dictionary.";
const strPreferencesPrefix = "extensions.dictoPop.";
const strLanguagesStatusbarId = "dp-languages-statusbar";
const strLanguageImageExtension = ".png";

const strConsoleID = strExtPrefix + "Console";


function dictoPop ( strSrcLanguage, strTargetLanguage, strDictionary, strTemplatePath )
{
	this.m_bActive = true;
	this.m_bShowExamplesImmediately = false; // Pons only
	this.m_bShowEntryImmediately = true;
	
	// Event data
	this.m_rangeParent = null;
	this.m_rangeOffset = -1;
	this.m_movedObject = null;
	this.m_movedOffsetX = 0;
	this.m_movedObjectY = 0;
	this.m_clientX = 0;
	this.m_clientY = 0;
	
	// Timeout ids and times
	this.m_loadDataTimeoutId = null;
	this.m_messageTimeoutId = null;
	this.m_toggleActiveTimeoutId = null;
	this.m_loadDataTimeout = 15000;
	this.m_messageTimeout = 3000;
	this.m_toggleActiveTimeout = 250;
	
	this.m_strings = null;
	this.m_xmlHttp = null;
	
	// Paths
	this.m_strTemplatePath = strTemplatePath ? strTemplatePath : defTemplatePath;
	this.m_strIconPath = this.m_strTemplatePath + "icons/";
	this.m_strCssPath = this.m_strTemplatePath + "css/";
	this.m_nControlsInitialized = 0;
	this.m_debugMsgCount = 0;
	
	this.m_dictionaries = new dictionaries ( strSrcLanguage, strTargetLanguage, strDictionary );
	this.getSetPreferences ( false );
	this.addRemoveEventListeners ( this.m_bActive );
	this.addRemoveEventListener ( EL_ON_SELECT, true /* bAdd */ );
}


dictoPop.prototype.abortLoadData = function ()
{
	this.clearTimeouts ();

	if ( this.m_xmlHttp )
	{
		this.m_xmlHttp.abort ();
		this.m_xmlHttp = null;
	}
}

		
dictoPop.prototype.loadData = function ( range, strUrl, strAnchorId )
{
	this.abortLoadData ();
	
	var strSearch = "";

	if ( strUrl == null )
	{
		if ( range )
			strSearch = range.toString ();
	
		if ( ! strSearch.length )
			return;

		strSearch = strSearch.toLowerCase ();
		strUrl = this.m_dictionaries.getUrl ( strSearch );
	}
	else
		strSearch = strUrl;

	// Language not supported
	if ( strUrl.length == 0 )
	{
		this.displayMessage ( strSearch, this.m_messageTimeout );
		return;
	}

	dump ( "loadData: " + strUrl + ", " + strSearch + "\n" );

	this.m_xmlHttp = createHttpRequest ();

	if ( this.m_xmlHttp == null )
		return;

	// Display the search progress
	this.displayMessage ( strSearch, 0, "searching" /* mode */ );

	// Callback function
	with ( this )
	{
		m_xmlHttp.onreadystatechange = function ()
		{
			if ( m_xmlHttp.readyState == 4 && m_xmlHttp.status == 200 )
			{
				var strResult = m_xmlHttp.responseText;
				var anchor = getDoc().getElementById ( strAnchorId );
				var strCreationTime;
				
				if ( anchor == null )
				{
					strCreationTime = new String ( new Date().getTime() );
					strAnchorId = new String ( strExtAnchorId + strCreationTime );
				}
				else
				{
					strCreationTime = strAnchorId.substr ( strExtAnchorId.length );
					
					var div = getAnchorElement ( anchor );
				
					// Destroy an existing dictionary box. The user might want to 
					// get another entry at the same anchor
					if ( div != null )
						div.parentNode.removeChild ( div );
				}

				var strDivId = strExtId + strCreationTime;

				if ( strResult.length == 0 ||
					! createDiv (
						strDivId,
						strResult,
						m_dictionaries.getStyleSheet (),
						strSearch ) )
				{
					// If there is no such entry display it too
					displayMessage ( strSearch, m_messageTimeout );
				}
				else
				{
					clearTimeouts ();
					createAnchor ( strDivId, strAnchorId, range );
				}
			}
		}
	}

	try
	{
		this.m_xmlHttp.open ( "GET", strUrl, true /* bAsynchronous */ );
		this.m_xmlHttp.send ( null );

		with ( this )
		{
			m_loadDataTimeoutId = 
				window.setTimeout ( 
					function () 
					{
						abortLoadData ();
						displayMessage ( "", m_messageTimeout, "abortLoading" );
					},
					m_loadDataTimeout );
		}
	}
	catch ( e )
	{
		this.abortLoadData ();
		
		debugMsgBox ( e );
	}
}

const eventListenerElementsPerEntry = 4;
const EL_ON_MOVE = 0;
const EL_ON_MOVE_END = 1;
const EL_ON_SELECT = 2;

var eventListeners = [
	// Do not change position of the first elements EL_xxx !
	// |type		|callback												capture|temporary
	"mousemove", 	function ( e ) { gDictoPop.onMoveObject ( e ); },		true, 	true,
	"mouseup", 		function ( e ) { gDictoPop.onMouseUp ( e ); },			true,	true,
	"select", 		function ( e ) { gDictoPop.onSelect ( e ); },			false, 	false,
	"mousedown", 	function ( e ) { gDictoPop.onMouseDown ( e ); },		false, 	true,
	"click", 		function ( e ) { gDictoPop.onClick ( this, e ); },		false, 	true,
	"keydown", 		function ( e ) { gDictoPop.onKeyDown ( e ); },			false,	true
];	


dictoPop.prototype.addRemoveEventListeners = function ( bAdd )
{
	for ( var i = 0; i < eventListeners.length / eventListenerElementsPerEntry; i++ )
		this.addRemoveEventListener ( i, bAdd, true /* bSkipCaptureListeners */ );
}

	
dictoPop.prototype.addRemoveEventListener = function ( index, bAdd, bSkipCaptureListener )
{
	var i = index * eventListenerElementsPerEntry;
	var strEvent = eventListeners [ i ];
	var bCapture = eventListeners [ i + 2 ];
	var bTemporary = eventListeners [ i + 3 ];
	var bSkipListener = bCapture && bSkipCaptureListener != null && bSkipCaptureListener == true;
	
	if ( ! bAdd )
	{
		// Do never remove non-temporary listeners (e.g. 'select'. Otherwise we don't have 
		// any opportunity to distinguish between Xul documents and Html documents.
		if ( bTemporary )
 			window.removeEventListener ( strEvent, eventListeners [ i + 1 ], bCapture );
	}
	
	else if ( ! bSkipListener )
		window.addEventListener ( strEvent, eventListeners [ i + 1 ], bCapture );
}

	
dictoPop.prototype.getCssStyles = function ( strId, strHtml, strStyleSheetName, range )
{
	var cssStyles = this.m_dictionaries.getCssStyleUrls ();
	var strCssStyles = "";
	
	cssStyles.unshift ( strExtStyles );

	for ( var i = 0; i < cssStyles.length; i++ )
		strCssStyles += getContents ( this.m_strCssPath + cssStyles [ i ], "text/css" );
	
	return strCssStyles;
}


dictoPop.prototype.createDiv = function ( strDivId, strHtml, strStyleSheetName, strSearch )
{
	var tempDiv;
	
	try
	{
		tempDiv = getDoc ().createElement ( "div" );
		tempDiv.innerHTML = strHtml;
	}
	catch ( e )
	{
		debugMsgBox ( e );
		return null;
	}

	var strCssStyles = this.getCssStyles ();
	var xsltProcessor = new XSLTProcessor ();
	var xmlDoc = getDoc ().implementation.createDocument ( "", "", null );
	
	xmlDoc.appendChild ( xmlDoc.importNode ( tempDiv, true /* deep copy */ ) );
	
	var strUrlBase = this.m_dictionaries.getUrlBase();
	var strDefaultParameters = [
		"showExamplesText",			this.getString ( "dp.labels.showExamples" ),
		"hideExamplesText",			this.getString ( "dp.labels.hideExamples" ),
		"showExamplesImmediately", 	this.m_bShowExamplesImmediately ? "1" : "0",
		"divId", 					strDivId,
		"cssStyles", 				strCssStyles,
		"baseUrl", 					"http://" + strUrlBase,
		"baseUrlName", 				strUrlBase,
		"searchText", 				strSearch ];

	for ( var i = 0; i < strDefaultParameters.length; i += 2 )
	{
		xsltProcessor.setParameter (
			null /* default name space */,
			strDefaultParameters [ i ], 
			strDefaultParameters [ i + 1 ] );
	}

	for ( var i = 4; i + 1 < arguments.length; i += 2 )
	{
		xsltProcessor.setParameter (
			null /* default name space */,
			arguments [ i ],
			arguments [ i + 1 ] );
	}
	xsltProcessor.importStylesheet ( getContents ( this.m_strTemplatePath + strStyleSheetName ) );

	var docFragment = xsltProcessor.transformToFragment ( xmlDoc, getDoc () );

	if ( ! docFragment || ! docFragment.hasChildNodes () )
	{
		docFragment = null;
		return null;
	}

	docFragment.id = strDivId;
 	getDoc ().body.appendChild ( docFragment );

	var div = getDoc ().getElementById ( strDivId );

	moveObject ( div, this.m_clientX, this.m_clientY );

	return div;
}


dictoPop.prototype.createAnchor = function ( strDivId, strAnchorId, range )
{
	var div = getDoc().getElementById ( strDivId );
	
	if ( ! div )
		return;
		
	try
	{
		var anchor = getDoc().getElementById ( strAnchorId );

		if ( anchor == null )
		{
			if ( range == null )
				return;
				
			anchor = getDoc().createElement ( "span" );
			
			var strStyle = 
				"text-decoration:underline;" + 
				"color:#002984; cursor: pointer;" + 
				"background-color: #00F000;";
				
			anchor.setAttribute ( "name", strExtAnchor );
			anchor.setAttribute ( "style", strStyle );
			anchor.setAttribute ( "id", strAnchorId );

			range.surroundContents ( anchor );
			range.detach ();
		}
		
		var boundingRect = anchor.getBoundingClientRect ();
		var anchorPos = getElementPosition ( anchor );

		moveObject (
			div, 
			anchorPos.x + ( boundingRect.right - boundingRect.left ),
			anchorPos.y + ( boundingRect.bottom - boundingRect.top ) );
	
		if ( ! this.m_bShowEntryImmediately )
			div.style.display = "none";
	}
	catch ( e )
	{
		debugMsgBox ( e );
	}
}


dictoPop.prototype.displayMessage = function ( strSearch, timeout, strMode )
{
	this.removeMessage();

	var div = this.createDiv (
						strExtMessageId,
						null /* strHtml */,
						"Msg.xsl",
						strSearch,
						"notFound",
						this.getString ( "dp.msg.notFound" ),
						"noConnection",
						this.getString ( "dp.msg.noConnection" ),
						"mode", 	
						strMode == null ? "" : strMode );

	with ( this )
	{
		if ( timeout != 0 )
			m_messageTimeoutId = window.setTimeout ( function () { removeMessage (); }, timeout );
	}
}


dictoPop.prototype.removeMessage = function ()
{
	this.dump ( "remove message\n" );
	
	while ( true )
	{
		var message = getDoc ().getElementsByName ( strExtMessage ).item ( 0 );
		
		if ( message == null )
			break;
			
		message = message.parentNode.removeChild ( message );
		message = null;
	}
}


dictoPop.prototype.clearTimeouts = function ()
{
	this.removeMessage();

	if ( this.m_messageTimeoutId )
	{
		window.clearTimeout ( this.m_messageTimeoutId );
		this.m_messageTimeoutId = null;
	}
	
	if ( this.m_loadDataTimeoutId )
	{
		window.clearTimeout ( this.m_loadDataTimeoutId );
		this.m_loadDataTimeoutId = null;
	}
	
	if ( this.m_toggleActiveTimeoutId )
	{
		window.clearTimeout ( this.m_toggleActiveTimeoutId );
		this.m_toggleActiveTimeoutId = null;
	}
}
	

dictoPop.prototype.initControl = 
function ( 
	strListId, 
	strCurrValue,
	items, 
	strMapPrefix, 
	strImagePattern,
	strOnCommand
)
{
	var list = getXul().getElementById ( strListId );
	
	if ( list == null || list.hidden == true )
		return 0;
		
	// The MAC implementation of Firefox fire the 'onload' only if there's at least one
	// image in one of the items.
	if ( list.itemCount == 1 && list.getItemAtIndex( 0 ).getAttribute ( "value" ) == "dummy" )
		list.removeAllItems ();
		
	var sorter = new Array;
	
	for ( prop in items )
	{
		try
		{		
			var item = {
					value : 	prop,
					oncommand : strOnCommand.replace ( /{value}/g, prop ),
					label : 	this.getString ( strMapPrefix + prop ),
					image :		strImagePattern.replace ( /{value}/g, prop ),
			}
				
			sorter.push ( item );
		}
		catch ( e )
		{
			debugMsgBox ( e );
		}
	}

	sorter.sort ( 
		function (a,b) 
			{ return a.label < b.label ? -1 : a.label > b.label ? 1 : 0; } );
	
	for ( var i = 0; i < sorter.length; i++ )
	{
		var sortItem = sorter [ i ];
		var listItem = list.appendItem ( sortItem.label, sortItem.value );
		
		listItem.setAttribute ( "oncommand", sortItem.oncommand );
		listItem.setAttribute ( "image", this.m_strIconPath + sortItem.image );
		
		if ( listItem.getAttribute ( "value" ) == strCurrValue )
			list.selectedItem = listItem;
	}
	
	return 1;
}


dictoPop.prototype.initControls = function ()
{
	if ( this.m_nControlsInitialized >= 3 ) // 3 controls to be initialized
		return;
		
	this.dump ( "init controls\n" );
	
	var nControlsInitialized = 0;

	nControlsInitialized += this.initControl ( 
									"SrcLanguagesSelection",
									this.m_dictionaries.getSrcLanguage (),
									this.m_dictionaries.getList ( LT_SRC_LANGUAGES ),
									strLanguageMapPrefix,
									"{value}" + strLanguageImageExtension,
									"gDictoPop.onSelectLanguage(true,'{value}');" );
							
	nControlsInitialized += this.initControl ( 
									"TargetLanguagesSelection", 
									this.m_dictionaries.getTargetLanguage (),
									this.m_dictionaries.getList ( LT_TARGET_LANGUAGES ),
									strLanguageMapPrefix,
									"{value}" + strLanguageImageExtension,
									"gDictoPop.onSelectLanguage(false,'{value}');" );
			
	nControlsInitialized += this.initControl ( 
									"DictionariesSelection", 
									this.m_dictionaries.getDictionary (),
									this.m_dictionaries.getList ( LT_DICTIONARIES ),
									strDictionaryMapPrefix,
									"books.gif",
									"gDictoPop.onSelectDictionary('{value}');" );
			
	this.onSelectLanguage ( true, this.m_dictionaries.getSrcLanguage () );
	
	if ( nControlsInitialized == 3 )
		this.m_nControlsInitialized = nControlsInitialized;
}


dictoPop.prototype.onMoveObject = function ( event )
{
	moveObject ( 
		this.m_movedObject, 
		event.pageX + this.m_movedOffsetX, 
		event.pageY + this.m_movedOffsetY );
		
	event.stopPropagation ();
	event.preventDefault ();
	
	this.dumpEvent ( null, event, " ** onMoveObject! ** " );
}


dictoPop.prototype.onMouseDown = function ( event )
{
	this.dumpEvent ( null, event, "** onMouseDown! ** " );

	if ( isValidTarget ( event.target ) )
	{
		var titlebar = findParentByClassName ( event.target, strExtTitleBar );
		var div = findParentOrSelfByPrefix ( titlebar, strExtId );
		
		if ( div )
		{
			this.addRemoveEventListener ( EL_ON_MOVE, true /* bAdd */ );

			var elemPos = getElementPosition ( div );
			
			this.m_movedOffsetX = elemPos.x - event.pageX;
			this.m_movedOffsetY = elemPos.y - event.pageY;
			this.m_movedObject = div;
		}
		else
			this.m_movedObject = null;

		this.addRemoveEventListener ( EL_ON_MOVE_END, true /* bAdd */ );
	}
}


dictoPop.prototype.onMouseUp = function ( event )
{	
	this.dumpEvent ( null, event, "** onMouseUp! ** " );

	this.addRemoveEventListener ( EL_ON_MOVE, false /* bAdd */ );
	this.addRemoveEventListener ( EL_ON_MOVE_END, false /* bAdd */ );
	
	// At the end of a moving process m_movedObject is not null
	if ( this.m_movedObject != null )
		this.m_movedObject = null;
		
	// If the user clicked into one of our elements (prefixed by 'strExtPrefix')
	// stop any further propagation. Otherwise we might have an embedded foreign
	// link executed
	else if ( findParentOrSelfByPrefix ( event.target, strExtPrefix ) )
	{
		event.stopPropagation ();
		event.preventDefault ();
	}
	else
	{
		// Prevent the loading of data if the user clicked into an already loaded object
		strSelection = this.getSelectionString ();
		
		if ( strSelection.length > 0 )
		{
			this.m_clientX = event.pageX;
			this.m_clientY = event.pageY;

			this.loadData ( this.getRange() );
		}
	}
}


dictoPop.prototype.onKeyDown = function ( event )
{
	this.dumpEvent ( null, event );

	if ( event.keyCode == event.DOM_VK_ESCAPE && isValidTarget ( event.target ) )
	{
		this.onRemoveAnchors (); // clears all timers
	}
}


dictoPop.prototype.onSelect = function ( event )
{
	this.dumpEvent ( null, event );

	var statusPanel = getXul().getElementById ( "dp-statusbar" );
	var bXulDocument = getDoc () == null || getDoc().getElementsByName == null;

	if ( bXulDocument )
		this.addRemoveEventListeners ( false /* bAdd */ );
	else
		this.addRemoveEventListeners ( this.m_bActive );

	statusPanel.hidden = bXulDocument;
}


dictoPop.prototype.onClick = function ( node, event )
{
	this.dumpEvent ( node, event );

	if ( ! isValidTarget ( event.target ) )
		return;
		
	if ( event.detail == 1 )
	{
		var clickedParent = findParentOrSelfByPrefix ( event.target, strExtPrefix );

		if ( ! clickedParent )
		{
			if ( event.target.tagName.toLowerCase () != 'a' )
			{
				this.m_rangeParent = event.rangeParent;
				this.m_rangeOffset = event.rangeOffset;
				this.m_clientX = event.pageX;
				this.m_clientY = event.pageY;

				this.loadData ( this.getRange () );
			}
		}
		else 
		{
			var bStopPropagation = true;
			
			// Display the div that contains the relevant information. We didn't handle
			// that case in the element itself since on an inline 'onclick' handler
			// we cannot stop further propagation of that event.
			if ( isTypeOf ( clickedParent, strExtAnchorId ) )
			{
				var strId = strExtId + clickedParent.id.substr ( strExtAnchorId.length );
				
				var div = getDoc().getElementById ( strId );

				if ( div )
				{
					if ( div.style.display == "none" )
					{
						div.parentNode.appendChild ( div );
						div.style.display = 'inline';
					}
					else
						div.style.display = 'none';
				}
			}
			
			// Click on the anchor to the dictionary's home page? If so
			// let the event pass.
			else if ( findParentByClassName ( event.target, strExtTitleBar ) )
				bStopPropagation = event.target.tagName.toLowerCase () != 'a';

			else if ( isTypeOf ( clickedParent, strExtId ) )
			{
				var anchor = findParentOrSelfByTagName ( event.target, "a", clickedParent.id );

				// If the user clicked into on an open dictionary box let this be
				// the top most one.
				if ( ! anchor )
				{
					if ( clickedParent.parentNode )
						clickedParent.parentNode.appendChild ( clickedParent );
				}
					
				// Click into an anchor in an open dictionary box. We interpret this
				// as the attempt to get further information into the same open
				// box
				else if ( anchor.href )
				{
					var strUrl = anchor.href;
					var strAnchorId = 
							strExtAnchorId + 
							clickedParent.id.substr ( strExtId.length );
							
					this.loadData ( null /* range */, strUrl, strAnchorId );
				}
			}
			
			if ( bStopPropagation )
			{
				event.stopPropagation ();
				event.preventDefault ();
			}
		}
	}
}


dictoPop.prototype.getLanguageString = function ( strLanguageId )
{
	return this.getString ( strLanguageMapPrefix + strLanguageId );
}


dictoPop.prototype.getString = function ( strId )
{
	var strResult = "";
	
	try
	{
		if ( this.m_strings == null )
			this.m_strings = getXul().getElementById ( "dp-strings" );
		
		strResult = this.m_strings.getString ( strId );
	}
	catch ( e )
	{
		debugMsgBox ( "Error: String '" + strId + "' not found\n\n" + e );
	}
	finally 
	{
		return strResult;
	}
}


dictoPop.prototype.getRange = function ()
{
	var strSelection = this.getSelectionString ();

	if ( strSelection.length > 0 )
	{
		var selection = this.getSelection ();
		var range = getDoc ().createRange ();
			
		range = selection.getRangeAt ( 0 );
		
		return range;
	}
		
	return this.getWordRangeFromEvent ( this.m_rangeParent, this.m_rangeOffset );
}


dictoPop.prototype.getAnchorElement = function ( anchor )
{
	var strAnchorId = new String ( anchor.id );
	var strDivId = strExtId + strAnchorId.replace ( strExtAnchorId, "" );
	
	return getDoc().getElementById ( strDivId );
}


dictoPop.prototype.removeAnchors = function ()
{
	while ( true )
	{
		var anchors = getDoc().getElementsByName ( strExtAnchor );

		if ( anchors.length == 0 )
			break;

		var anchor = anchors [ 0 ];
		var anchorParent = anchor.parentNode;
		var div = this.getAnchorElement ( anchor );

		if ( div && div.parentNode )
			div.parentNode.removeChild ( div );

		if ( anchorParent )
		{
			while ( anchor.firstChild )
				anchorParent.insertBefore (
					anchor.removeChild ( anchor.firstChild ),
					anchor );
			
			anchorParent.removeChild ( anchor );
			
			// The operations above do not merge adjacent text nodes. So
			// do it now.
			anchorParent.normalize ();
		}
	}
}


dictoPop.prototype.getSelection = function ()
{
	if ( window.document.commandDispatcher.focusedWindow )
		return window.document.commandDispatcher.focusedWindow.getSelection();
	else
		return null;
}


dictoPop.prototype.getSelectionString = function ()
{
	var strSelection = "";
	
	try
	{
		var selection = this.getSelection ();
		
		if ( selection != null && selection.rangeCount > 0 )
			strSelection = selection.toString();
	}
	catch ( e )
	{
	}
	
	return strSelection;
}


dictoPop.prototype.onHelp = function ()
{
	getDoc().location.href = defTemplatePath + "help.html";
}


dictoPop.prototype.onCheckMenu = function ()
{
	var anchorsCount = getDoc().getElementsByName ( strExtAnchor ).length;
	var disabled = anchorsCount == 0;
	var menuItem;
	
	menuItem = getXul().getElementById ( "dp-cmd-showAnchors" );
	menuItem.disabled = disabled;
	menuItem = getXul().getElementById ( "dp-cmd-hideAnchors" );
	menuItem.disabled = disabled;
	menuItem = getXul().getElementById ( "dp-cmd-removeAnchors" );
	menuItem.disabled = disabled;
}


dictoPop.prototype.onToolTipCurrDictionaryValues = function ()
{
	getXul().getElementById ( "ttCurrSrcLanguage" ).value = 
		this.getLanguageString ( this.m_dictionaries.getSrcLanguage () );
		
	getXul().getElementById ( "ttCurrTargetLanguage" ).value = 
		this.getLanguageString ( this.m_dictionaries.getTargetLanguage () );
	
	getXul().getElementById ( "ttCurrDictionary" ).value = 
		this.getString ( strDictionaryMapPrefix + this.m_dictionaries.getDictionary () );
		
	getXul().getElementById ( "ttStartStop" ).value = 
		this.getString ( this.m_bActive ? "dp.tooltip.stopDictoPop" : "dp.tooltip.startDictoPop" );
}


dictoPop.prototype.onSelectLanguage = function ( bSrcLanguage, strLanguage )
{
	var strSrcLanguage;
	var strTargetLanguage = null;
	var strDictionary;
	this.dump ( "onSelectLanguage: bSrcLanguage==" + bSrcLanguage + ", " + strLanguage + "\n" ); 
	this.clearTimeouts ();

	if ( bSrcLanguage )
	{
		strSrcLanguage = strLanguage;
		strTargetLanguage = this.selectSrcLanguageDependants ( 
									"TargetLanguagesSelection", 
									strSrcLanguage, 
									strTargetLanguage, 
									LT_TARGET_LANGUAGES );
	}

	else
	{
		strSrcLanguage = this.m_dictionaries.getSrcLanguage ();
		strTargetLanguage = strLanguage;
	}
		
	strDictionary = this.selectSrcLanguageDependants ( 
							"DictionariesSelection", 
							strSrcLanguage, 
							strTargetLanguage, 
							LT_DICTIONARIES );

	this.setDictionaryValues ( strSrcLanguage, strTargetLanguage, strDictionary );
	this.updateCurrSrcLanguageImage ();
}						


dictoPop.prototype.selectSrcLanguageDependants = function ( strListId, strSrcLanguage, strTargetLanguage, nListType )
{
	var list = getXul().getElementById ( strListId );
	
	if ( list == null )
		return;
		
	var availableItems = this.m_dictionaries.getList ( nListType, strSrcLanguage, strTargetLanguage );
	var currValue = list.selectedItem ? list.selectedItem.getAttribute ( "value" ) : "undefined";
	var firstVisibleItem = null;
	var visibleItems = 0;

	for ( var i = 0; i < list.itemCount; i++ )
	{
		var currItem = list.getItemAtIndex ( i );
		var currItemValue = currItem.getAttribute ( "value" );
		var bHidden = ! availableItems [ currItemValue ];
		
		currItem.hidden = bHidden;
		
		if ( ! currItem.hidden )
			visibleItems++;
		
		if ( ! bHidden && firstVisibleItem == null )
			firstVisibleItem = currItem;
	}
	
	if ( ! availableItems [ currValue ] && firstVisibleItem )
		list.selectedItem = firstVisibleItem;
		
	list.disabled = visibleItems == 0;
	
	return list.selectedItem ? list.selectedItem.getAttribute ( "value" ) : "";
}


dictoPop.prototype.onSelectDictionary = function ( strDictionary )
{
	this.clearTimeouts ();
	this.setDictionaryValues ( null /* use current */, null /* use current */, strDictionary );
}


dictoPop.prototype.setDictionaryValues = function ( strSrcLanguage, strTargetLanguage, strDictionary )
{
	if ( this.m_dictionaries.setDictionaryValues ( strSrcLanguage, strTargetLanguage, strDictionary ) )
		this.getSetPreferences ( true );
}


dictoPop.prototype.onRemoveAnchors = function ()
{
	this.clearTimeouts ();
	this.removeAnchors ();
}


dictoPop.prototype.onShowHideAnchors = function ( bShow )
{
	this.clearTimeouts ();
	
	var anchors = getDoc().getElementsByName ( strExtAnchor );

	for ( var i = 0; i < anchors.length; i++ )
		this.getAnchorElement ( anchors [ i ] ).style.display = bShow ? "block" : "none";
}


dictoPop.prototype.showHideStatusbar = function ( show )
{
	this.clearTimeouts();
	
	var statusBar = getXul().getElementById ( strLanguagesStatusbarId );
	
	statusBar.hidden = ! show;
	
	this.updateCurrSrcLanguageImage();
}


/**
	\brief Der Knopf, der dictoPop startet oder stoppt, soll auch die Einstellungen einblenden, wenn
	ein Doppelklick mit der linken Maustaste erfolgt. Leider feuert Mozilla in einem solchen Fall
	beide Ereignisse: onclick und ondblclick. Daher wird bei dem Versuch, dictoPop zu starten/stoppen
	zunächst ein Timer eingeschaltet, der in der Methode showHideStatusbar wieder gelöscht wird.
*/
dictoPop.prototype.onToggleActive = function ()
{
	this.clearTimeouts ();
	
	with ( this )
	{
		m_toggleActiveTimeoutId = 
			window.setTimeout ( 
				function () 
				{
					m_bActive = ! m_bActive;
					dump ( "onToggleActive=" + m_bActive );

					addRemoveEventListeners ( m_bActive );
					
					if ( m_bActive == false )
						onShowHideAnchors ( false /* bShow */ );
						
					updateCurrSrcLanguageImage ();
					getSetPreferences ( true );

					var console = getDoc().getElementById ( strConsoleID );
					
					if ( console != null )
						console.hidden = ! m_bActive;	
				},
				m_toggleActiveTimeout );
	}
}


dictoPop.prototype.updateCurrSrcLanguageImage = function ()
{
	var imgCurrSrcLanguage = getXul().getElementById ( "imgCurrSrcLanguage" );
	
	if ( imgCurrSrcLanguage )
	{
		var strImageName = this.m_dictionaries.getSrcLanguage () + strLanguageImageExtension;
		
		if ( ! this.m_bActive )
			strImageName = "-" + strImageName;
			
		imgCurrSrcLanguage.src = this.m_strIconPath + strImageName;
		
		this.dump ( "updateCurrSrcLanguageImage: " + imgCurrSrcLanguage.src + "\n" );
		
		imgCurrSrcLanguage.hidden = ! getXul().getElementById ( strLanguagesStatusbarId ).hidden;
	}
}


dictoPop.prototype.dumpEvent = function ( node, event, strMoreInfo )
{
	var strSearch = this.getSelectionString ();

	if ( strSearch.length != 0 )
		strSearch =
			"search {'" +
			strSearch +
			"'," +
			this.m_dictionaries.getSrcLanguage () +
			":" +
			this.m_dictionaries.getTargetLanguage () + "}, ";

	var strText = new String;

	try
	{
		if ( event.target.textContent != null )
		{
			var _strText = String ( event.target.textContent );

			if ( _strText.length > 20 )
				_strText = _strText.substr ( 0, 20 ) + "[..]";

			strText = "TEXT{" + _strText + "},";
		}
		var range = this.getWordRangeFromEvent ( event.rangeParent, event.rangeOffset );
		
		if ( range )
		{
			strText = strText + ";WORT{" + range.toString () + "} ";
			range.detach ();
		}
	}

	catch ( e )
	{
		debugMsgBox ( e );
	}

	var strId = new String ( event.target.id );

	if ( strId.length )
		strId = "id:" + strId + ", ";

				
	var strMovedObject = "";
	
	if ( this.m_movedObject )
		strMovedObject = ", MovedObject{" + this.m_movedObject.id + "}";

	strText = strSearch = "";
	
	if ( strMoreInfo == null )
		strMoreInfo = new String;

	var strCancelable = ", event.cancelable{bool}".replace ( "bool", event.cancelable );
	var nodeInfo = "";
	
	if ( node != null )
		nodeInfo = ", nodeInfo: " + node;
	
	this.dump ( 
		event + 
		strMoreInfo +
		event.type.toUpperCase () + ":" + 
		strText +
		strSearch +
		"target {" +
		strId +
		"&lt;" +
		event.target.tagName +
		"&gt;}, " + 
		"xy {" +
		event.pageX +
		":" +
		event.pageY +
		"}, event.detail{" +
		event.detail + 
		"}, event.button{" + 
		event.button + 
		"}, OTarget{" + 
		event.originalTarget + 
		", " + 
		event.originalTarget.id + "}" + 
		strMovedObject + 
		strCancelable + 
		"\n" );
}


dictoPop.prototype.dump = function ( args )
{
	if ( gDebug == false )
		return;
		
	try
	{
		var console = getDoc().getElementById ( strConsoleID );
		
		if ( console == null )
		{
			console = getDoc().createElement ( "div" );
			
			if ( console != null )
			{
				console.setAttribute ( "id", strConsoleID );
				console.setAttribute ( "style", "width: 3000px; height: 300px; font-family: 'Courier New'; font-size: 12px; border: 1px solid black; background: white; position: absolute; right: 0; top: 0; overflow: auto;"); // tescht
				
				getDoc().body.appendChild ( console );
			}
		}
		
		if ( console != null )
		{
			this.m_debugMsgCount++;
			
			var p = getDoc().createElement ( "p" );
			
			p.innerHTML = "<b>" + this.m_debugMsgCount + "&nbsp;</b>" + args;

			if ( console.firstChild != null )
				console.insertBefore ( p, console.firstChild );
			else
				console.appendChild ( p );
		}
	}
	catch ( e )
	{
		debugMsgBox ( e );
	}
}


dictoPop.prototype.getSetPreferences = function ( bSetPreferences )
{
	try
	{
		var preferences = Components.classes [ "@mozilla.org/preferences-service;1" ].
								getService ( Components.interfaces.nsIPrefBranch );
								
		if ( bSetPreferences )
		{
			preferences.setIntPref ( 
				strPreferencesPrefix + "messageTimeout", 
				this.m_messageTimeout );

			preferences.setIntPref ( 
				strPreferencesPrefix + "loadDataTimeout", 
				this.m_loadDataTimeout );

			preferences.setIntPref ( 
				strPreferencesPrefix + "toggleActiveTimeout", 
				this.m_toggleActiveTimeout );

			preferences.setBoolPref ( 
				strPreferencesPrefix + "active", 
				this.m_bActive );

			preferences.setBoolPref ( 
				strPreferencesPrefix + "showEntryImmediately", 
				this.m_bShowEntryImmediately );

			preferences.setBoolPref ( 
				strPreferencesPrefix + "showExamplesImmediately", 
				this.m_bShowExamplesImmediately );

			preferences.setCharPref ( 
				strPreferencesPrefix + "srcLanguage", 
				this.m_dictionaries.getSrcLanguage () );
							
			preferences.setCharPref ( 
				strPreferencesPrefix + "targetLanguage", 
				this.m_dictionaries.getTargetLanguage () );
							
			preferences.setCharPref ( 
				strPreferencesPrefix + "dictionary", 
				this.m_dictionaries.getDictionary () );
		}
		else
		{
			this.m_messageTimeout = 
				preferences.getIntPref ( strPreferencesPrefix + "messageTimeout" );
					
			this.m_loadDataTimeout = 
				preferences.getIntPref ( strPreferencesPrefix + "loadDataTimeout" );
					
			this.m_toggleActiveTimeout = 
				preferences.getIntPref ( strPreferencesPrefix + "toggleActiveTimeout" );
					
			this.m_bActive = 
				preferences.getBoolPref ( strPreferencesPrefix + "active" );
					
			this.m_bShowEntryImmediately = 
				preferences.getBoolPref ( strPreferencesPrefix + "showEntryImmediately" );
					
			this.m_bShowExamplesImmediately = 
				preferences.getBoolPref ( strPreferencesPrefix + "showExamplesImmediately" );
					
			this.m_dictionaries.setDictionaryValues ( 
				preferences.getCharPref ( strPreferencesPrefix + "srcLanguage" ),
				preferences.getCharPref ( strPreferencesPrefix + "targetLanguage" ),
				preferences.getCharPref ( strPreferencesPrefix + "dictionary" ) );
		}
	}
	catch ( e )
	{
		debugMsgBox ( e );
	}
}


dictoPop.prototype.getWordRangeFromEvent = function ( rangeParent, rangeOffset )
{
	if ( ! rangeParent )
		return null;
		
	var range;
	var bError = true;

	try
	{
		range = getDoc ().createRange ();
		
		try
		{
			range.setStart ( rangeParent, rangeOffset );
			range.setEnd ( rangeParent, rangeOffset );
			
			var strWord = expandRangeToWord ( range );
			
			bError = strWord.length == 0;
		}
		catch ( e )
		{
			debugMsgBox ( e );
		}
	}
	catch ( e )
	{
		debugMsgBox ( e );
		return null;
	}

	if ( bError )
	{
		range.detach ();
		range = null;
	}

	return range;
}
