//------------------------------------------------------------------------------
// Switch on/off debugging
//------------------------------------------------------------------------------
gDebug = false;

//------------------------------------------------------------------------------
// Helpers
//------------------------------------------------------------------------------

function getDoc ()
{
	return content.document;
}


function getXul ()
{
	return document;
}


function moveObject ( node, xPos, yPos )
{
	if ( node )
	{
		node.style.left = xPos + "px";
		node.style.top = yPos + "px";
	}
}


function findParentOrSelfByPrefix ( node, strParentIdPrefix )
{
	while ( node != null )
	{
		if ( node.id && node.id.indexOf ( strParentIdPrefix ) != -1 )
			return node;

		node = node.parentNode;
	}
	
	return null;
}


function findParentOrSelfByTagName ( node, strTagName, strStopNodeId )
{
	strTagName = strTagName.toLowerCase ();
	
	while ( node != null )
	{
		if ( node.tagName.toLowerCase () == strTagName )
			return node;
			
		else if ( node.id == strStopNodeId )
			return null;

		node = node.parentNode;
	}
	
	return null;
}


function findParentByClassName ( node, strClassName )
{
	while ( node != null )
	{
		if ( node.getAttribute && node.getAttribute ( "class" ) == strClassName )
			return node;

		node = node.parentNode;
	}

	return null;
}


function isTypeOf ( node, strClassNamePrefix )
{
	return node != null && node.id.indexOf ( strClassNamePrefix ) == 0;
}


function dumpProps ( object )
{
	var props = new Array;
	var strProps = "";
	var nProps = 0;
	
	for ( prop in object )
	{
		strProps += String ( ++nProps ) + ". " + prop;
		
		if ( nProps % 2 == 0 )
			strProps += "\n";
		else
			strProps += "\t\t\t";
	}	
	return "Properties: " + nProps + "\n" + strProps;
}


function showParents ( node )
{
	var strResult = "";
	
	if ( node )
	{
		strResult += "\n" + node + ",type:" + typeof node;

		if ( node.name )
			strResult += ": name=" + node.name + ",";

		if ( node.id )
			strResult += ": id=" + node.id + ",";
			
		if ( node.style && node.style.zIndex )
			strResult += ": z-Index=" + node.style.zIndex + ",";
			
		strResult += " " + showParents ( node.parentNode );
	}
	
	return strResult;
}


function showChildren ( parent )
{
	var strResult = "";
	
	if ( parent )
	{
		for ( var i = 0; i < parent.childNodes.length; i++ )
		{
			var node = parent.childNodes [ i ];
			
			strResult += "\n" + node;

			if ( node.name )
				strResult += ": name=" + node.name + ",";

			if ( node.id )
				strResult += ": id=" + node.id + ",";
				
			if ( node.style && node.style.zIndex )
				strResult += ": z-Index=" + node.style.zIndex + ",";
		}
	}
	
	return strResult;
}


const invalidTagNames = [ "INPUT", "BUTTON", "TEXTAREA" ];

function isValidTarget ( node )
{
	if ( isContentElement ( node ) )
	{
		for ( var i = 0; i < invalidTagNames.length; i++ )
		{
			if ( invalidTagNames [ i ] == node.tagName )
				return false;
		}
		
		return true;
	}
	
	return false;
}


function isContentElement ( node )
{
	if ( node == null )
		return false;
		
	else if ( node == getDoc() )
		return true;
		
	return isContentElement ( node.parentNode );
}


function expandRangeToWord ( range )
{
	var wordPattern = /.*[\s,;'"!.:].*/;

	var nStart = range.startOffset;
	var nEnd = range.endOffset;
	
	while ( nStart > 0 )
	{
		try
		{
			range.setStart ( range.startContainer, nStart - 1 );
			
			if ( wordPattern.test ( range.toString () ) )
				break;
			
			nStart--;
		}
		catch ( e )
		{
			break;
		}
	}

	range.setStart ( range.startContainer, nStart );
	
	while ( true )
	{
		try
		{
			range.setEnd ( range.endContainer, nEnd + 1 );
			
			if ( wordPattern.test ( range.toString () ) )
				break;
			
			nEnd++;
		}
		catch ( e )
		{
			break;
		}
	}

	range.setEnd ( range.endContainer, nEnd );
	
	return range.toString();
}


function getElementPosition ( elem )
{
	var posX = 0;
	var posY = 0;
              
	while ( elem )
	{
		posX += elem.offsetLeft;
		posY += elem.offsetTop;
		elem = elem.offsetParent;
	}
                        		      
   return { x:posX, y: posY };
}


function debugMsgBox ( strMsg )
{
	if ( gDebug === true )
		alert ( strMsg );
}


function createHttpRequest ()
{
	var httpRequest = null;

	try
	{
		httpRequest = new XMLHttpRequest();
	}
	catch ( e )
	{
		debugMsgBox ( e );
	}

	if ( ! httpRequest || httpRequest.readyState != 0 )
		return null;

	return httpRequest;
}


function getContents ( strUrl, mimeType )
{
	var httpRequest = createHttpRequest ();

	if ( httpRequest )
	{
		try
		{
			httpRequest.open ( "GET", strUrl, false /* asynchronous */ );
			
			if ( mimeType )
				httpRequest.overrideMimeType ( mimeType );
				
			httpRequest.send ( null );

			if ( mimeType )
				return httpRequest.responseText;
				
			return httpRequest.responseXML;
		}
		catch ( e )
		{
			debugMsgBox ( "getContents(" + strUrl + ")\n" + e );
		}
	}

	return null;
}


function getTag ( strIn, strSearchTag, bSlice )
{
	var nTagStart = strIn.search ( new RegExp ( strSearchTag, "gim" ) );

	if ( nTagStart == -1 )
		return "";

	var nTagEnd = nTagStart;
	var strTag = strSearchTag.replace ( new RegExp ( "([^<]*)<(\\w+)\\b([\\s\\S]*)", "gi" ), "$2" );
	var nEndTagLen = strTag.length + 3; // "</" + ">"
	var reTag = new RegExp ( "(<" + strTag + ")|(</" + strTag + ">)", "gim" );
	var nStack = 0;
	var reResult;

	reTag.lastIndex = nTagStart;

	while ( ( reResult = reTag.exec ( strIn ) ) )
	{
		var nTagLen = reTag.lastIndex - reResult.index + 1;

		if ( nTagLen < nEndTagLen )
			nStack++;
		else
		{
			nStack--;
			nTagEnd = reTag.lastIndex;
		}

		if ( nStack == 0 )
			break;
	}

	var strResult = "";

	if ( nStack == 0 )
	{
		if ( bSlice == null || bSlice == false )
			strResult =  strIn.substring ( nTagStart, nTagEnd );
		else
			strResult = strIn.substring ( 0, nTagStart ) + strIn.substring ( nTagEnd );
	}

	return strResult;
}


function stringToClipboard ( strOut )
{
	var oClipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);  
		
	oClipboard.copyString ( strOut );
 }
 